import bpy
import tempfile
import os


def new_material(material_name):
    material = bpy.data.materials.new(name=material_name)
    material.shadow_method = "NONE"
    material.blend_method = "BLEND"
    material.use_nodes = True
    bsdf = material.node_tree.nodes["Principled BSDF"]
    bsdf.inputs["Specular IOR Level"].default_value = 0.0
    return material, bsdf


def new_image_node(material, image):
    image_node = material.node_tree.nodes.new("ShaderNodeTexImage")
    image_node.image = image
    image_node.interpolation = "Closest"
    return image_node


def new_rgb_node(material, color):
    color_node = material.node_tree.nodes.new("ShaderNodeRGB")
    color_node.outputs["Color"].default_value = color
    return color_node


def new_value_node(material, value):
    value_node = material.node_tree.nodes.new("ShaderNodeValue")
    value_node.outputs["Value"].default_value = value
    return value_node


def new_color_mix_node(material, clamp=False):
    mix_node = material.node_tree.nodes.new("ShaderNodeMixRGB")
    mix_node.blend_type = "MULTIPLY"
    mix_node.inputs["Fac"].default_value = 1.0
    mix_node.use_clamp = clamp
    return mix_node


def new_vertex_color_node(material):
    vertex_color_node = material.node_tree.nodes.new("ShaderNodeVertexColor")
    return vertex_color_node


def new_object_info_node(material):
    value_mix_node = material.node_tree.nodes.new("ShaderNodeObjectInfo")
    return value_mix_node


def new_value_divide_1024_node(material):
    math_node = material.node_tree.nodes.new("ShaderNodeMath")
    math_node.operation = "DIVIDE"
    math_node.inputs[1].default_value = 1024.0
    return math_node


def new_value_mix_node(material, clamp=False):
    math_node = material.node_tree.nodes.new("ShaderNodeMath")
    math_node.operation = "MULTIPLY"
    math_node.use_clamp = clamp
    return math_node


class Materials:
    def __init__(self, map_images):
        images = []
        with tempfile.TemporaryDirectory() as tmp_dir:
            for i, image in enumerate(map_images):
                image_path = os.path.join(tmp_dir, f"{i}_{image.name}.png")
                image.save(image_path)
                b_image = bpy.data.images.load(filepath=image_path)
                b_image.pack()
                b_image.filepath = ""
                images.append(b_image)
        self.images = images
        self.quad_materials = {}
        self.tiles_layer_materials = {}

    def quad_material(self, image_index):
        if image_index not in self.quad_materials:
            if image_index is None:
                material, bsdf = new_material("Quad untextured")

                vertex_color_node = new_vertex_color_node(material)
                object_info_node = new_object_info_node(material)

                divide_node = new_value_divide_1024_node(material)
                material.node_tree.links.new(divide_node.inputs[0], object_info_node.outputs["Object Index"])

                color_mix_node = new_color_mix_node(material, clamp=True)
                material.node_tree.links.new(color_mix_node.inputs["Color1"], vertex_color_node.outputs["Color"])
                material.node_tree.links.new(color_mix_node.inputs["Color2"], object_info_node.outputs["Color"])

                value_mix_node = new_value_mix_node(material, clamp=True)
                material.node_tree.links.new(value_mix_node.inputs[0], vertex_color_node.outputs["Alpha"])
                material.node_tree.links.new(value_mix_node.inputs[1], divide_node.outputs["Value"])

                material.node_tree.links.new(bsdf.inputs["Base Color"], color_mix_node.outputs["Color"])
                material.node_tree.links.new(bsdf.inputs["Alpha"], value_mix_node.outputs["Value"])
            else:
                material, bsdf = new_material(f"Quad {self.images[image_index].name}")

                image_node = new_image_node(material, self.images[image_index])
                vertex_color_node = new_vertex_color_node(material)

                first_color_mix_node = new_color_mix_node(material)
                material.node_tree.links.new(first_color_mix_node.inputs["Color1"], image_node.outputs["Color"])
                material.node_tree.links.new(first_color_mix_node.inputs["Color2"], vertex_color_node.outputs["Color"])

                first_alpha_mix_node = new_value_mix_node(material)
                material.node_tree.links.new(first_alpha_mix_node.inputs[0], image_node.outputs["Alpha"])
                material.node_tree.links.new(first_alpha_mix_node.inputs[1], vertex_color_node.outputs["Alpha"])

                object_info_node = new_object_info_node(material)
                divide_node = new_value_divide_1024_node(material)
                material.node_tree.links.new(divide_node.inputs[0], object_info_node.outputs["Object Index"])

                second_color_mix_node = new_color_mix_node(material, clamp=True)
                material.node_tree.links.new(second_color_mix_node.inputs["Color1"], first_color_mix_node.outputs["Color"])
                material.node_tree.links.new(second_color_mix_node.inputs["Color2"], object_info_node.outputs["Color"])

                second_value_mix_node = new_value_mix_node(material, clamp=True)
                material.node_tree.links.new(second_value_mix_node.inputs[0], first_alpha_mix_node.outputs["Value"])
                material.node_tree.links.new(second_value_mix_node.inputs[1], divide_node.outputs["Value"])

                material.node_tree.links.new(bsdf.inputs["Base Color"], second_color_mix_node.outputs["Color"])
                material.node_tree.links.new(bsdf.inputs["Alpha"], second_value_mix_node.outputs["Value"])
            self.quad_materials[image_index] = material
        return self.quad_materials[image_index]

    def tiles_layer_material(self, color, image_index):
        if (image_index, color) not in self.tiles_layer_materials:
            if image_index is None:
                material, bsdf = new_material(f"Tiles {color}")

                rgb_node = new_rgb_node(material, [v / 255 for v in color])
                alpha_node = new_value_node(material, color[3] / 255)
                object_info_node = new_object_info_node(material)

                divide_node = new_value_divide_1024_node(material)
                material.node_tree.links.new(divide_node.inputs[0], object_info_node.outputs["Object Index"])

                color_mix_node = new_color_mix_node(material, clamp=True)
                material.node_tree.links.new(color_mix_node.inputs["Color1"], rgb_node.outputs["Color"])
                material.node_tree.links.new(color_mix_node.inputs["Color2"], object_info_node.outputs["Color"])

                alpha_mix_node = new_value_mix_node(material, clamp=True)
                material.node_tree.links.new(alpha_mix_node.inputs[0], alpha_node.outputs["Value"])
                material.node_tree.links.new(alpha_mix_node.inputs[1], divide_node.outputs["Value"])

                material.node_tree.links.new(bsdf.inputs["Base Color"], color_mix_node.outputs["Color"])
                material.node_tree.links.new(bsdf.inputs["Alpha"], alpha_mix_node.outputs["Value"])
            else:
                material, bsdf = new_material(f"Tiles {self.images[image_index].name} {color}")

                image_node = new_image_node(material, self.images[image_index])
                rgb_node = new_rgb_node(material, [v / 255 for v in color])
                alpha_node = new_value_node(material, color[3] / 255)

                first_color_mix_node = new_color_mix_node(material)
                material.node_tree.links.new(first_color_mix_node.inputs["Color1"], image_node.outputs["Color"])
                material.node_tree.links.new(first_color_mix_node.inputs["Color2"], rgb_node.outputs["Color"])

                first_alpha_mix_node = new_value_mix_node(material)
                material.node_tree.links.new(first_alpha_mix_node.inputs[0], image_node.outputs["Alpha"])
                material.node_tree.links.new(first_alpha_mix_node.inputs[1], alpha_node.outputs["Value"])

                object_info_node = new_object_info_node(material)
                divide_node = new_value_divide_1024_node(material)
                material.node_tree.links.new(divide_node.inputs[0], object_info_node.outputs["Object Index"])

                second_color_mix_node = new_color_mix_node(material, clamp=True)
                material.node_tree.links.new(second_color_mix_node.inputs["Color1"], first_color_mix_node.outputs["Color"])
                material.node_tree.links.new(second_color_mix_node.inputs["Color2"], object_info_node.outputs["Color"])

                second_value_mix_node = new_value_mix_node(material, clamp=True)
                material.node_tree.links.new(second_value_mix_node.inputs[0], first_alpha_mix_node.outputs["Value"])
                material.node_tree.links.new(second_value_mix_node.inputs[1], divide_node.outputs["Value"])

                material.node_tree.links.new(bsdf.inputs["Base Color"], second_color_mix_node.outputs["Color"])
                material.node_tree.links.new(bsdf.inputs["Alpha"], second_value_mix_node.outputs["Value"])
            self.quad_materials[(image_index, color)] = material
        return self.quad_materials[(image_index, color)]
